/*
 * Copyright AnalyseSI, 2008
 * 
 * Loic DREUX
 * 
 * Ce logiciel est régi par la licence CeCILL soumise au droit français et
 * respectant les principes de diffusion des logiciels libres. Vous pouvez
 * utiliser, modifier et/ou redistribuer ce programme sous les conditions
 * de la licence CeCILL telle que diffusée par le CEA, le CNRS et l'INRIA 
 * sur le site "http://www.cecill.info".
 * 
 * En contrepartie de l'accessibilité au code source et des droits de copie,
 * de modification et de redistribution accordés par cette licence, il n'est
 * offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
 * seule une responsabilité restreinte pèse sur l'auteur du programme,  le
 * titulaire des droits patrimoniaux et les concédants successifs.
 * 
 * A cet égard  l'attention de l'utilisateur est attirée sur les risques
 * associés au chargement,  à l'utilisation,  à la modification et/ou au
 * développement et à la reproduction du logiciel par l'utilisateur étant 
 * donné sa spécificité de logiciel libre, qui peut le rendre complexe à 
 * manipuler et qui le réserve donc à des développeurs et des professionnels
 * avertis possédant  des  connaissances  informatiques approfondies.  Les
 * utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
 * logiciel à leurs besoins dans des conditions permettant d'assurer la
 * sécurité de leurs systèmes et ou de leurs données et, plus généralement, 
 * à l'utiliser et l'exploiter dans les mêmes conditions de sécurité. 
 * 
 * Le fait que vous puissiez accéder à cet en-tête signifie que vous avez 
 * pris connaissance de la licence CeCILL, et que vous en avez accepté les
 * termes.
 */
package org.eucalyptus.analysesi;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JFrame;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;

import org.eucalyptus.analysesi.db.DbManager;
import org.eucalyptus.analysesi.db.DbPropertiesLocation;
import org.eucalyptus.analysesi.gui.AnalyseSIFrame;
import org.eucalyptus.analysesi.plugin.PluginManager;
import org.eucalyptus.analysesi.reverse.ReverseManager;
import org.eucalyptus.analysesi.save.SaveManager;
import org.eucalyptus.egt.AbstractApplication;
import org.eucalyptus.egt.Application;
import org.eucalyptus.egt.JEFrame;
import org.hibiscus.version.ReleaseStage;
import org.hibiscus.version.VersionFactory;
import org.jvnet.substance.skin.SubstanceBusinessLookAndFeel;
import org.jvnet.substance.skin.SubstanceRavenGraphiteLookAndFeel;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * L'application AnalyseSI.
 * 
 * @author Loic Dreux
 */
public class AnalyseSIApplication extends AbstractApplication implements
		DbPropertiesLocation {
	/**
	 * Le logger.
	 */
	private static Logger logger = LoggerFactory
			.getLogger(AnalyseSIApplication.class);

	/**
	 * La méthode d'entrée du programme.
	 * 
	 * @param args
	 *            les arguments à fournir au programme
	 */
	public static void main(final String... args) {
		// Lance l'application
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				Application application = new AnalyseSIApplication();
				application.run(args);
			}
		});
	}

	/**
	 * La liste des plugins
	 */
	private List<File> pluginsFolders;

	/**
	 * Le constructeur de {@link AnalyseSIApplication}.
	 * 
	 * @param splashScreen
	 *            Le splashscreen
	 * @param debug
	 *            Active le mode debug
	 */
	public AnalyseSIApplication() {
		super("AnalyseSI", "Hibiscus", VersionFactory.buildVersion(0, 7,
				ReleaseStage.NIGHTLY));
	}

	/**
	 * Retourne le gestionnaire de base de données.
	 * 
	 * @return le gestionnaire de base de données
	 */
	public DbManager getDbManager() {
		return (DbManager) getManager(DbManager.class);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eucalyptus.egt.Application#getMainFrame()
	 */
	@Override
	public JEFrame<? extends Application> getMainFrame() {
		return new AnalyseSIFrame(this, getSplashScreen());
	}

	/**
	 * Retourne le gestionnaire de plugin.
	 * 
	 * @return le gestionnaire de plugin
	 */
	public PluginManager getPluginManager() {
		PluginManager pluginManager = (PluginManager) getManager(PluginManager.class);
		logger.debug("PluginManager: " + pluginManager.toString());
		return pluginManager;
	}

	/**
	 * Retourne la liste des répertoires qui contiennent les plugins.
	 * 
	 * @return la liste des répertoires qui contiennent les plugins
	 */
	public List<File> getPluginsFolders() {
		return pluginsFolders;
	}

	/**
	 * Retourne le gestionnaire de {@link ReverseManager}.
	 * 
	 * @return le gestionnaire de {@link ReverseManager}
	 */
	public ReverseManager getReverseManager() {
		return (ReverseManager) getManager(ReverseManager.class);
	}

	/**
	 * Retourne le gestionnaire de {@link SaveManager}.
	 * 
	 * @return le gestionnaire de {@link SaveManager}
	 */
	public SaveManager getSaveManager() {
		return (SaveManager) getManager(SaveManager.class);
	}

	/**
	 * Initialise la liste des répertoires qui contienne des plugins.
	 */
	private void initPluginsFolder() {
		this.pluginsFolders = new ArrayList<File>();

		// Création du répertoire des plugins par défaut
		File defaultPluginsFolder = new File(this.getSettingsDirectory()
				.getAbsolutePath()
				+ File.separator + "plugins");
		if (!defaultPluginsFolder.isDirectory()) {
			defaultPluginsFolder.mkdirs();
		}

		pluginsFolders.add(defaultPluginsFolder);
	}

	/**
	 * Initialise les propriétés liés à swing.
	 */
	private void initUISettings() {
		try {
			JFrame.setDefaultLookAndFeelDecorated(true);
			//UIManager.setLookAndFeel("org.jvnet.substance.SubstanceLookAndFeel");
			//UIManager.setLookAndFeel("org.jvnet.substance.skin.SubstanceBusinessBlueSteelLookAndFeel");
			//SubstanceBusinessBlackSteelLookAndFeel.set(new org.jvnet.substance.watermark.SubstanceStripeWatermark());
			UIManager.setLookAndFeel(new SubstanceRavenGraphiteLookAndFeel());
			
			// Désactivation du log pour Flying Saucer
			System.setProperty("xr.util-logging.loggingEnabled", "false");
		} catch (Exception e) {
			logger.error(e.getLocalizedMessage(), e);
		}
	}

	public void run(String... args) {
		// Initialisation des propriétés Swing
		this.initUISettings();

		// On ajoute les resources
		this.getResourcesProvider().addResource(DbPropertiesLocation.class,
				this);

		// Initialisation du SplashScreen
		getSplashScreen().setProgress(0);

		// Initialise les répertoire qui contiennent les plugins
		this.initPluginsFolder();
		
		// Chargemenment des plugins
		getPluginManager().loadPlugins(this.getPluginsFolders());

		// Création de la fenêtre AnalyseSI
		JEFrame<?> analyseSIFrame = getMainFrame();

		// On affiche AnalyseSIFrame et on cache le SplashScreen
		getSplashScreen().setProgress(100);
		analyseSIFrame.setVisible(true);
		getSplashScreen().setVisible(false);
	}
}