/*
 * Copyright AnalyseSI, 2008
 * 
 * Loic DREUX
 * 
 * Ce logiciel est régi par la licence CeCILL soumise au droit français et
 * respectant les principes de diffusion des logiciels libres. Vous pouvez
 * utiliser, modifier et/ou redistribuer ce programme sous les conditions
 * de la licence CeCILL telle que diffusée par le CEA, le CNRS et l'INRIA 
 * sur le site "http://www.cecill.info".
 * 
 * En contrepartie de l'accessibilité au code source et des droits de copie,
 * de modification et de redistribution accordés par cette licence, il n'est
 * offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
 * seule une responsabilité restreinte pèse sur l'auteur du programme,  le
 * titulaire des droits patrimoniaux et les concédants successifs.
 * 
 * A cet égard  l'attention de l'utilisateur est attirée sur les risques
 * associés au chargement,  à l'utilisation,  à la modification et/ou au
 * développement et à la reproduction du logiciel par l'utilisateur étant 
 * donné sa spécificité de logiciel libre, qui peut le rendre complexe à 
 * manipuler et qui le réserve donc à des développeurs et des professionnels
 * avertis possédant  des  connaissances  informatiques approfondies.  Les
 * utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
 * logiciel à leurs besoins dans des conditions permettant d'assurer la
 * sécurité de leurs systèmes et ou de leurs données et, plus généralement, 
 * à l'utiliser et l'exploiter dans les mêmes conditions de sécurité. 
 * 
 * Le fait que vous puissiez accéder à cet en-tête signifie que vous avez 
 * pris connaissance de la licence CeCILL, et que vous en avez accepté les
 * termes.
 */
package org.eucalyptus.analysesi.db;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.sql.Connection;
import java.sql.Driver;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eucalyptus.egt.ManagerContainer;
import org.eucalyptus.egt.ResourcesProvider;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.thoughtworks.xstream.XStream;

/**
 * Le gestionnaire de base de données.
 * 
 * @author Loic Dreux
 */
public class DefaultDbManager implements DbManager {
	// Le logger
	private static final Logger logger = LoggerFactory
			.getLogger(DbManager.class);

	/**
	 * La liste des drivers jdbc pris en charge.
	 */
	private List<Class<?>> jdbcDriversList;

	/**
	 * La correspondance driver -> url
	 */
	private Map<Class<?>, String> urlStarts;

	/**
	 * La correspondante driver -> sqlTestQuery
	 */
	private Map<Class<?>, String> sqlTestQueries;

	/**
	 * L'objet XStream pour le chargement des connections paramètrées.
	 */
	private XStream xstream;
	
	/**
	 * Permet de récupérer l'endroit où est stoqué le fichier de propriétés.
	 */
	private DbPropertiesLocation dbPropertiesLocation;

	/**
	 * Le constructeur du gestionnaire de base de données.
	 */
	public DefaultDbManager(ManagerContainer managerContainer, ResourcesProvider resourcesProvider) {
		// Instanciation des propriétés
		this.jdbcDriversList = new ArrayList<Class<?>>();
		this.urlStarts = new HashMap<Class<?>, String>();
		this.sqlTestQueries = new HashMap<Class<?>, String>();
		this.dbPropertiesLocation = (DbPropertiesLocation)resourcesProvider.getResource(DbPropertiesLocation.class);
	} 

	/*
	 * Ajoute un nouveau driver Jdbc.
	 * 
	 * @param driver Le driver
	 * 
	 * @param urlStart Le début de l'url
	 */
	@Override
	public void addJdbcDriver(Class<?> driver, String urlStart,
			String sqlTestQuery) {
		try {
			jdbcDriversList.add(driver);
			urlStarts.put(driver, urlStart);
			sqlTestQueries.put(driver, sqlTestQuery);
		} catch (Exception e) {
			logger.error(e.getLocalizedMessage(), e);
		}
	}

	/*
	 * Retourne la liste des drivers JDBC pris en charge.
	 * 
	 * @return la liste des drivers JDBC pris en charge
	 */
	@Override
	public List<Class<?>> getJdbcDrivers() {
		return jdbcDriversList;
	}

	/*
	 * sqlTestQueries Retourne le début de l'url selon le driver.
	 * 
	 * @param driver Le driver
	 * 
	 * @return le début de l'url selon le driver
	 */
	@Override
	public String getUrlStart(Class<?> driver) {
		return urlStarts.get(driver);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eucalyptus.analysesi.db.DbManager#testConnection(org.eucalyptus.analysesi
	 * .db.DbConnectionParameters)
	 */
	@Override
	public boolean testConnection(DbConnectionParameters connectionParameters) {
		try {
			Class<?> class1 = getJdbcDriver(connectionParameters);
			Driver driver = (Driver)class1.newInstance();
			DriverManager.registerDriver(driver);
			Connection connection = DriverManager.getConnection(
					connectionParameters.getUrl(), connectionParameters
							.getUser(), connectionParameters.getPassword());
			Statement statement = connection.createStatement();
			ResultSet result = statement
					.executeQuery(getSqlTestQuery(connectionParameters
							.getClass()));
			return result.getInt(0) == 1;
		} catch (Exception e) {
			logger.warn(e.getLocalizedMessage(), e);
			return false;
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eucalyptus.analysesi.db.DbManager#getSqlTestQuery(java.lang.Class)
	 */
	@Override
	public String getSqlTestQuery(Class<?> driver) {
		return sqlTestQueries.get(driver);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eucalyptus.analysesi.db.DbManager#getConfiguredConnections()
	 */
	@SuppressWarnings("unchecked")
	@Override
	public List<DbConnectionParameters> getConfiguredConnections() {
		try {
			// La sauvegarde des connections paramètrées se fait avec XStream.
			XStream xstream = buildXStream();
			File file = new File(dbPropertiesLocation.getSettingsDirectory().getAbsoluteFile()
					+ File.separator + "connections.xml");
			if (file.exists()) {
				InputStream input = new FileInputStream(file);
				return (List<DbConnectionParameters>) xstream.fromXML(input);
			} else {
				return new ArrayList<DbConnectionParameters>();
			}
		} catch (IOException e) {
			logger.error(e.getLocalizedMessage(), e);
			return new ArrayList<DbConnectionParameters>();
		}
	}

	/* 
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eucalyptus.analysesi.db.DbManager#saveConfiguredConnections(java.
	 * util.List)
	 */
	@Override
	public void saveConfiguredConnections(
			List<DbConnectionParameters> configuredConnections) {
		try {
			// La sauvegarde des connections paramètrées se fait avec XStream.
			XStream xstream = buildXStream();
			File file = new File(dbPropertiesLocation.getSettingsDirectory()
					+ File.separator + "connections.xml");
			OutputStream output = new FileOutputStream(file);
			xstream.toXML(configuredConnections, output);
		} catch (IOException e) {
			logger.error(e.getLocalizedMessage(), e);
		}
	}

	/**
	 * Fabrique d'objets XStream spécialement paramétrés pour la sauvegarde
	 * d'objet {@link DbConnectionParameters}.
	 * 
	 * @return un objet XStream paramétré
	 */
	private XStream buildXStream()
	{
		// On vérifie que XStream a déjà été instancié
		if (xstream != null)
			return xstream;

		// Création de l'instance et paramètrage de l'objet
		xstream = new XStream();
		xstream.alias("connection", DbConnectionParameters.class);
		return xstream;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eucalyptus.analysesi.db.DbManager#getJdbcDriver(org.eucalyptus.analysesi
	 * .db.DbConnectionParameters)
	 */
	@Override
	public Class<?> getJdbcDriver(DbConnectionParameters connection) {
		for(Class<?> driver: jdbcDriversList) {
			if (driver.getName().equals(connection.getDriver())) {
				return driver;
			}
		}
		return null;
	}
}
