/*
 * Copyright AnalyseSI, 2008
 * 
 * Loic DREUX
 * 
 * Ce logiciel est régi par la licence CeCILL soumise au droit français et
 * respectant les principes de diffusion des logiciels libres. Vous pouvez
 * utiliser, modifier et/ou redistribuer ce programme sous les conditions
 * de la licence CeCILL telle que diffusée par le CEA, le CNRS et l'INRIA 
 * sur le site "http://www.cecill.info".
 * 
 * En contrepartie de l'accessibilité au code source et des droits de copie,
 * de modification et de redistribution accordés par cette licence, il n'est
 * offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
 * seule une responsabilité restreinte pèse sur l'auteur du programme,  le
 * titulaire des droits patrimoniaux et les concédants successifs.
 * 
 * A cet égard  l'attention de l'utilisateur est attirée sur les risques
 * associés au chargement,  à l'utilisation,  à la modification et/ou au
 * développement et à la reproduction du logiciel par l'utilisateur étant 
 * donné sa spécificité de logiciel libre, qui peut le rendre complexe à 
 * manipuler et qui le réserve donc à des développeurs et des professionnels
 * avertis possédant  des  connaissances  informatiques approfondies.  Les
 * utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
 * logiciel à leurs besoins dans des conditions permettant d'assurer la
 * sécurité de leurs systèmes et ou de leurs données et, plus généralement, 
 * à l'utiliser et l'exploiter dans les mêmes conditions de sécurité. 
 * 
 * Le fait que vous puissiez accéder à cet en-tête signifie que vous avez 
 * pris connaissance de la licence CeCILL, et que vous en avez accepté les
 * termes.
 */
package org.eucalyptus.analysesi.gui.db;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;

import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JToolBar;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.eucalyptus.analysesi.db.DbConnectionParameters;
import org.eucalyptus.analysesi.db.DbManager;
import org.eucalyptus.egt.factory.ActionFactory;
import org.eucalyptus.egt.loader.ImageLoader;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Le composant graphique représentant la liste des connections configurées.
 * 
 * @author Loic Dreux
 */
public class ConfiguredConnectionsList extends JPanel implements
		ConnectionsListener {
	Logger logger = LoggerFactory.getLogger(ConfiguredConnectionsList.class);

	/**
	 * Le gestionnaire de base de données.
	 */
	private DbManager dbManager;

	/**
	 * La liste des listeners du composant.
	 */
	private List<ConnectionsListener> connectionsListeners;

	/**
	 * Un éditeur de paramètres de connection.
	 */
	private ConnectionEditor connectionEditor;

	/**
	 * Le modèle des connections.
	 */
	private ConnectionsModel connectionsModel;

	/*
	 * ============================================= Les Actions
	 */

	/**
	 * Ajoute une connection.
	 */
	private Action add;

	/**
	 * Supprime une connection.
	 */
	private Action remove;

	/*
	 * ============================================= Les Composants
	 */

	private JList list;

	/**
	 * Le constructeur de {@link ConfiguredConnectionsList}.
	 * 
	 * @param dbManager
	 *            Le gestionnaire de base de données
	 */
	public ConfiguredConnectionsList(DbManager dbManager,
			ConnectionEditor connectionEditor) {
		super(new BorderLayout());
		this.dbManager = dbManager;
		this.connectionEditor = connectionEditor;
		this.connectionsListeners = new ArrayList<ConnectionsListener>();
		this.connectionsModel = new ConnectionsModel(dbManager);

		// Initialisation des actions
		this.initActions();

		// Construction du panel
		this.add(BorderLayout.NORTH, buildToolbar());
		this.add(BorderLayout.CENTER, buildPanel());
		this.setBorder(BorderFactory.createEmptyBorder(4, 4, 4, 4));
	}

	/**
	 * Ajoute un listener au composant.
	 * 
	 * @param connectionsListener
	 *            Le listener à ajouter
	 */
	public void addConnectionsListener(ConnectionsListener connectionsListener) {
		connectionsListeners.add(connectionsListener);
	}

	/**
	 * Fabrique le panel.
	 * 
	 * @return le panel
	 */
	private JPanel buildPanel() {
		JPanel p = new JPanel(new BorderLayout());

		// On fabrique la liste
		list = new JList(connectionsModel);
		list.addListSelectionListener(new ListHandler());
		list.setCellRenderer(new ConnectionsRenderer());
		p.add(BorderLayout.CENTER, new JScrollPane(list));

		return p;
	}

	/**
	 * Fabrique la toolbar.
	 * 
	 * @return la toolbar
	 */
	private JComponent buildToolbar() {
		JToolBar toolbar = new JToolBar();
		toolbar.add(add);
		toolbar.add(remove);
		toolbar.setBorder(BorderFactory.createEmptyBorder());
		return toolbar;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eucalyptus.analysesi.gui.db.ConnectionsListener#
	 * configuredConnectionsSaved
	 * (org.eucalyptus.analysesi.gui.db.ConnectionsEvent)
	 */
	@Override
	public void configuredConnectionsSaved(ConnectionsEvent e) {
		connectionsModel.fireContentsChanged(this, 0, connectionsModel
				.getSize());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eucalyptus.analysesi.gui.db.ConnectionsListener#connectionSelected
	 * (org.eucalyptus.analysesi.gui.db.ConnectionsEvent)
	 */
	@Override
	public void connectionSelected(ConnectionsEvent e) {
	}

	/**
	 * Retourne l'éditeur de paramètres de connection.
	 * 
	 * @return the connectionEditor
	 */
	public ConnectionEditor getConnectionEditor() {
		return connectionEditor;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.JComponent#getMaximumSize()
	 */
	@Override
	public Dimension getMaximumSize() {
		return new Dimension(200, (int) super.getMaximumSize().getHeight());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.JComponent#getPreferredSize()
	 */
	@Override
	public Dimension getPreferredSize() {
		return new Dimension(200, (int) super.getPreferredSize().getHeight());
	}

	/**
	 * Initialise les actions.
	 */
	private void initActions() {
		add = ActionFactory.buildAction("Ajouter",
				"Ajouter une nouvelle connection", Actions.ADD.name(),
				ImageLoader.getInstance().getIcon(
						"tango/16x16/actions/list-add.png"), new AddHandler());
		remove = ActionFactory.buildAction("Supprimer",
				"Supprimer une connection", Actions.REMOVE.name(), ImageLoader
						.getInstance().getIcon(
								"tango/16x16/actions/list-remove.png"),
				new RemoveHandler());
	}

	/**
	 * Supprime un listener au composant.
	 * 
	 * @param connectionsListener
	 *            le listener à supprimer
	 */
	public void removeConnectionsListener(
			ConnectionsListener connectionsListener) {
		connectionsListeners.remove(connectionsListener);
	}

	/**
	 * Modifie l'éditeur de paramètres de connection.
	 * 
	 * @param connectionEditor
	 *            the connectionEditor to set
	 */
	public void setConnectionEditor(ConnectionEditor connectionEditor) {
		this.connectionEditor = connectionEditor;
	}

	/**
	 * Les actions du composant
	 * 
	 * @author Loic Dreux
	 */
	private enum Actions {
		ADD, REMOVE
	}

	/**
	 * Ajoute une connection.
	 * 
	 * @author Loic Dreux
	 */
	private class AddHandler implements ActionListener {

		/*
		 * (non-Javadoc)
		 * 
		 * @see
		 * java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent
		 * )
		 */
		@Override
		public void actionPerformed(ActionEvent e) {
			logger.debug("Ajout d'une connection");

			// On récupère la connection
			DbConnectionParameters connection = connectionEditor
					.getDbConnectionParameters();

			// On l'ajoute à la liste des connections configurées
			List<DbConnectionParameters> configuredConnections = dbManager
					.getConfiguredConnections();
			configuredConnections.add(connection);

			// On sauvegarde
			dbManager.saveConfiguredConnections(configuredConnections);

			// On préviens tout le monde
			ConnectionsEvent connectionsEvent = new ConnectionsEvent(
					ConfiguredConnectionsList.this, configuredConnections);
			for (ConnectionsListener listener : connectionsListeners) {
				listener.configuredConnectionsSaved(connectionsEvent);
			}

			// On met à jour la liste
			connectionsModel.fireContentsChanged(
					ConfiguredConnectionsList.this, 0, configuredConnections
							.size());
		}

	}

	/**
	 * Supprime une connection.
	 * 
	 * @author Loic Dreux
	 */
	private class RemoveHandler implements ActionListener {

		/*
		 * (non-Javadoc)
		 * 
		 * @see
		 * java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent
		 * )
		 */
		@Override
		public void actionPerformed(ActionEvent e) {
			logger.debug("Suppression d'une connection");

			// On récupère la connection
			DbConnectionParameters connection = (DbConnectionParameters) list
					.getSelectedValue();

			// On l'ajoute à la liste des connections configurées
			List<DbConnectionParameters> configuredConnections = dbManager
					.getConfiguredConnections();
			configuredConnections.remove(connection);

			// On sauvegarde
			dbManager.saveConfiguredConnections(configuredConnections);

			// On préviens tout le monde
			ConnectionsEvent connectionsEvent = new ConnectionsEvent(
					ConfiguredConnectionsList.this, configuredConnections);
			for (ConnectionsListener listener : connectionsListeners) {
				listener.configuredConnectionsSaved(connectionsEvent);
			}

			// On met à jour la liste
			connectionsModel.fireContentsChanged(
					ConfiguredConnectionsList.this, 0, configuredConnections
							.size());
		}
	}

	/**
	 * Gère la sélection de la liste
	 * 
	 * @author Loic Dreux
	 */
	private class ListHandler implements ListSelectionListener {

		/*
		 * (non-Javadoc)
		 * 
		 * @see
		 * javax.swing.event.ListSelectionListener#valueChanged(javax.swing.
		 * event.ListSelectionEvent)
		 */
		@Override
		public void valueChanged(ListSelectionEvent e) {
			logger.debug("Selection d'une connection");
			// On préviens tout le monde
			ConnectionsEvent connectionsEvent = new ConnectionsEvent(
					ConfiguredConnectionsList.this, (DbConnectionParameters)list.getSelectedValue());
			for (ConnectionsListener listener : connectionsListeners) {
				listener.connectionSelected(connectionsEvent);
			}
		}
	}
}
