/*
 * Copyright AnalyseSI, 2007
 * 
 * Loic DREUX
 * 
 * Ce logiciel est régi par la licence CeCILL soumise au droit français et
 * respectant les principes de diffusion des logiciels libres. Vous pouvez
 * utiliser, modifier et/ou redistribuer ce programme sous les conditions
 * de la licence CeCILL telle que diffusée par le CEA, le CNRS et l'INRIA 
 * sur le site "http://www.cecill.info".
 * 
 * En contrepartie de l'accessibilité au code source et des droits de copie,
 * de modification et de redistribution accordés par cette licence, il n'est
 * offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
 * seule une responsabilité restreinte pèse sur l'auteur du programme,  le
 * titulaire des droits patrimoniaux et les concédants successifs.
 * 
 * A cet égard  l'attention de l'utilisateur est attirée sur les risques
 * associés au chargement,  à l'utilisation,  à la modification et/ou au
 * développement et à la reproduction du logiciel par l'utilisateur étant 
 * donné sa spécificité de logiciel libre, qui peut le rendre complexe à 
 * manipuler et qui le réserve donc à des développeurs et des professionnels
 * avertis possédant  des  connaissances  informatiques approfondies.  Les
 * utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
 * logiciel à leurs besoins dans des conditions permettant d'assurer la
 * sécurité de leurs systèmes et ou de leurs données et, plus généralement, 
 * à l'utiliser et l'exploiter dans les mêmes conditions de sécurité. 
 * 
 * Le fait que vous puissiez accéder à cet en-tête signifie que vous avez 
 * pris connaissance de la licence CeCILL, et que vous en avez accepté les
 * termes.
 */
package org.eucalyptus.analysesi.gui.dictionary;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JTextField;

import org.eucalyptus.analysesi.model.TypeAttribute;
import org.eucalyptus.analysesi.model.TypeAttributeProperty;
import org.jdesktop.swingx.combobox.EnumComboBoxModel;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Cette fabrique construit des composants à partir d'un type d'attribut en
 * utilisant l'introspection.
 * 
 * @author Loic Dreux
 */
public class PropertiesTypeAttributeComponentFactory {
	// Le logger
	private static Logger logger = LoggerFactory
			.getLogger(PropertiesTypeAttributeComponentFactory.class);

	/**
	 * Le cache des composants. 
	 */
	private static Map<Class<? extends TypeAttribute>, List<JComponent>> cacheComponents = new LinkedHashMap<Class<? extends TypeAttribute>, List<JComponent>>() {
		private static final int MAX_ENTRIES = 100;

		protected boolean removeEldestEntry(
				Map.Entry<Class<? extends TypeAttribute>, List<JComponent>> eldest) {
			return size() > MAX_ENTRIES;
		}
	};

	/**
	 * Le cache des labels.
	 */
	private static Map<Class<? extends TypeAttribute>, List<String>> cacheLabels = new LinkedHashMap<Class<? extends TypeAttribute>, List<String>>() {
		private static final int MAX_ENTRIES = 100;

		protected boolean removeEldestEntry(
				Map.Entry<Class<? extends TypeAttribute>, List<String>> eldest) {
			return size() > MAX_ENTRIES;
		}
	};

	/**
	 * Fabrique plusieurs composants à partir des propriétés d'un type
	 * d'attribut.
	 * 
	 * @param comboBoxTypeAttribute
	 * @return
	 */
	public static List<JComponent> buildComponents(
			Class<? extends TypeAttribute> typeAttributeClass) {
		// On vérifie dans le cache
		if (!cacheComponents.containsKey(typeAttributeClass))
			loadTypeAttribute(typeAttributeClass);

		// On réinitialise les objets
		for (JComponent component : cacheComponents.get(typeAttributeClass)) {
			if (component instanceof JTextField)
				((JTextField) component).setText("");
			if (component instanceof JComboBox)
				((JComboBox) component).setSelectedIndex(0);
		}

		// On retourne les objets
		return cacheComponents.get(typeAttributeClass);
	}

	/**
	 * @param typeAttributeClass
	 * @param componentsPropertiesTypeAttribute
	 * @return
	 */
	public static TypeAttribute buildTypeAttribute(
			Class<? extends TypeAttribute> typeAttributeClass,
			List<JComponent> componentsPropertiesTypeAttribute) {
		try {
			TypeAttribute typeAttribute = typeAttributeClass.newInstance();

			// On récupère la classe
			Field[] fields = typeAttributeClass.getDeclaredFields();

			// On parcours les champs
			int i = 0;
			for (Field field : fields) {
				TypeAttributeProperty a = field
						.getAnnotation(TypeAttributeProperty.class);

				if (a != null) {
					JComponent component = componentsPropertiesTypeAttribute
							.get(i);
					Object value = null;
					if (component instanceof JComboBox) {
						value = ((JComboBox) component).getSelectedItem();
					} else if (component instanceof JTextField) {
						value = ((JTextField) component).getText();
					} else {
						throw new IllegalArgumentException(
								"Le composant graphique n'est ni un JComboBox, ni un JTextField");
					}
					if (field.getType().equals(Integer.TYPE)) {
						value = Integer.valueOf((String) value);
					}
					logger.debug("Value of " + fields + ": " + value);
					field.setAccessible(true);
					field.set(typeAttribute, value);
				}
				i++;
			}

			return typeAttribute;
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
		}
		return null;
	}

	/**
	 * Retourne la listes des labels qui correspondent aux propriétés d'un type
	 * d'attribut.
	 * 
	 * @param comboBoxTypeAttribute
	 * @return
	 */
	public static List<String> getFieldsName(
			Class<? extends TypeAttribute> typeAttributeClass) {
		// On vérifie dans le cache
		if (!cacheLabels.containsKey(typeAttributeClass))
			loadTypeAttribute(typeAttributeClass);

		// On retourne les libellés
		return cacheLabels.get(typeAttributeClass);
	}

	/**
	 * On charge le type d'attribut dans le cache
	 * 
	 * @param typeAttribut
	 *            le type d'attribut à charger
	 */
	@SuppressWarnings("unchecked")
	private static void loadTypeAttribute(
			Class<? extends TypeAttribute> typeAttributeClass) {
		logger.debug("Chargement du type d'attribut : "
				+ typeAttributeClass.getName());

		// On récupère la classe
		Field[] fields = typeAttributeClass.getDeclaredFields();
		List<JComponent> components = new ArrayList<JComponent>();
		List<String> labels = new ArrayList<String>();

		// On parcours les champs
		for (Field field : fields) {
			TypeAttributeProperty a = field
					.getAnnotation(TypeAttributeProperty.class);
			if (a != null) {
				logger.debug("Propriété ajouté : " + field.getName());

				// Création du composants
				Class<?> classField = field.getType();
				if (classField.isEnum()) {
					components.add(new JComboBox(new EnumComboBoxModel(
							classField)));
				} else {
					components.add(new JTextField());
				}

				// Récupération du libellé
				labels.add(a.value() + " :");
			}
		}

		// On ajoute le résultat dans le cache et on le retourne
		cacheComponents.put(typeAttributeClass, components);
		cacheLabels.put(typeAttributeClass, labels);
	}
}