/*
 * Copyright AnalyseSI, 2007
 * 
 * Loic DREUX
 * 
 * Ce logiciel est régi par la licence CeCILL soumise au droit français et
 * respectant les principes de diffusion des logiciels libres. Vous pouvez
 * utiliser, modifier et/ou redistribuer ce programme sous les conditions
 * de la licence CeCILL telle que diffusée par le CEA, le CNRS et l'INRIA 
 * sur le site "http://www.cecill.info".
 * 
 * En contrepartie de l'accessibilité au code source et des droits de copie,
 * de modification et de redistribution accordés par cette licence, il n'est
 * offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
 * seule une responsabilité restreinte pèse sur l'auteur du programme,  le
 * titulaire des droits patrimoniaux et les concédants successifs.
 * 
 * A cet égard  l'attention de l'utilisateur est attirée sur les risques
 * associés au chargement,  à l'utilisation,  à la modification et/ou au
 * développement et à la reproduction du logiciel par l'utilisateur étant 
 * donné sa spécificité de logiciel libre, qui peut le rendre complexe à 
 * manipuler et qui le réserve donc à des développeurs et des professionnels
 * avertis possédant  des  connaissances  informatiques approfondies.  Les
 * utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
 * logiciel à leurs besoins dans des conditions permettant d'assurer la
 * sécurité de leurs systèmes et ou de leurs données et, plus généralement, 
 * à l'utiliser et l'exploiter dans les mêmes conditions de sécurité. 
 * 
 * Le fait que vous puissiez accéder à cet en-tête signifie que vous avez 
 * pris connaissance de la licence CeCILL, et que vous en avez accepté les
 * termes.
 */
package org.eucalyptus.analysesi.model;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import com.thoughtworks.xstream.annotations.XStreamAlias;

/**
 * La classe {@link MCD} permet de représenter un Modèle Conceptuel de Donnée grâce
 * à 5 caractéristiques : Entités, Propriétés, Identifiants, Associations et Cardinalités.
 * 
 * @author Loic Dreux
 */
@XStreamAlias("mcd")
public class MCD {
	/**
	 * La liste des entités
	 */
	@XStreamAlias("entities")
	private List<Entity> entities;

	/**
	 * La liste des relations entre entités
	 */
	@XStreamAlias("relations")
	private List<Relation> relations;
	
	/**
	 * Le dictionnaire des informations associé à ce MCD.
	 */
	@XStreamAlias("dictionary")
	private Dictionary dictionary;
	
	/**
	 * La liste des notes.
	 */
	@XStreamAlias("notes")
	private List<Note> notes;

	/**
	 * Le constructeur par défaut de la classe {@link MCD}.
	 */
	public MCD(Dictionary dictionary) {
		this.entities = new ArrayList<Entity>();
		this.relations = new ArrayList<Relation>();
		this.notes = new ArrayList<Note>();
		this.dictionary = dictionary;
	}

	/**
	 * Ajoute une entité au MCD.
	 * @param entity L'entité à ajouter au MCD
	 */
	public void addEntity(Entity entity) {
		entities.add(entity);
	}
	
	/**
	 * Retourne une entité du MCD
	 * @param index l'index de l'entité
	 * @return une entité du MCD
	 */
	public Entity getEntity(int index) {
		return entities.get(index);
	}
	
	/**
	 * Retourne le nombre d'entité présent dans le MCD.
	 * @return le nombre d'entité présent dans le MCD
	 */
	public int sizeEntities() {
		return entities.size();
	}

	/**
	 * Retourne la liste des entités sous forme d'Iterable.
	 * @return la liste des entités sous forme d'Iterable
	 */
	public Iterable<Entity> getEntities() {
		return new Iterable<Entity>() {
			public Iterator<Entity> iterator() {
				return entities.iterator();
			}	
		};
	}
	
	/**
	 * Retourne la liste des associations sous forme d'Iterable.
	 * @return la liste des associations sous forme d'Iterable
	 */
	public Iterable<Relation> getRelations() {
		return new Iterable<Relation>() {
			public Iterator<Relation> iterator() {
				return relations.iterator();
			}
		};
	}
	
	/**
	 * Supprime toutes les entités.
	 */
	public void clearEntities() {
		entities.clear();
	}


	/**
	 * Ajoute une relation au MCD.
	 * @param relation la relation à ajouter au MCD
	 */
	public void addRelation(Relation relation) {
		relations.add(relation);
	}
	
	/**
	 * Retourne le dictionnaire des informations.
	 * @return le dictionnaire des informations
	 */
	public Dictionary getDictionary() {
		return dictionary;
	}
	
	/**
	 * Ajoute une note.
	 * @param note la note à ajouter
	 */
	public void addNote(Note note) {
		notes.add(note);
	}
	
	/**
	 * Supprime une note.
	 * @param note la note à supprimer
	 */
	public void removeNote(Note note) {
		notes.remove(note);
	}
	
	/**
	 * Retourne une note.
	 * @param index l'index de la note
	 * @return une note
	 */
	public Note getNote(int index) {
		return notes.get(index);
	}
	
	/**
	 * Retourne la liste des notes.
	 * @return la liste des notes
	 */
	public Iterable<Note> getNotes() {
		return new Iterable<Note>() {
			public Iterator<Note> iterator() {
				return notes.iterator();
			}
		};
	}
	
	/**
	 * Retourne le nombre de note.
	 * @return le nombre de note
	 */
	public int sizeNotes() {
		return notes.size();
	}
	
	/**
	 * Supprime toute les notes.
	 */
	public void clearNotes() {
		notes.clear();
	}
}
