/*
 * Copyright AnalyseSI, 2008
 * 
 * Loic DREUX
 * 
 * Ce logiciel est régi par la licence CeCILL soumise au droit français et
 * respectant les principes de diffusion des logiciels libres. Vous pouvez
 * utiliser, modifier et/ou redistribuer ce programme sous les conditions
 * de la licence CeCILL telle que diffusée par le CEA, le CNRS et l'INRIA 
 * sur le site "http://www.cecill.info".
 * 
 * En contrepartie de l'accessibilité au code source et des droits de copie,
 * de modification et de redistribution accordés par cette licence, il n'est
 * offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
 * seule une responsabilité restreinte pèse sur l'auteur du programme,  le
 * titulaire des droits patrimoniaux et les concédants successifs.
 * 
 * A cet égard  l'attention de l'utilisateur est attirée sur les risques
 * associés au chargement,  à l'utilisation,  à la modification et/ou au
 * développement et à la reproduction du logiciel par l'utilisateur étant 
 * donné sa spécificité de logiciel libre, qui peut le rendre complexe à 
 * manipuler et qui le réserve donc à des développeurs et des professionnels
 * avertis possédant  des  connaissances  informatiques approfondies.  Les
 * utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
 * logiciel à leurs besoins dans des conditions permettant d'assurer la
 * sécurité de leurs systèmes et ou de leurs données et, plus généralement, 
 * à l'utiliser et l'exploiter dans les mêmes conditions de sécurité. 
 * 
 * Le fait que vous puissiez accéder à cet en-tête signifie que vous avez 
 * pris connaissance de la licence CeCILL, et que vous en avez accepté les
 * termes.
 */
package org.eucalyptus.analysesi.model;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 * Un "objet" Merise est la classe parente des objet Relation et Entity.
 * 
 * @author Loic Dreux
 */
public class MeriseObject {
	/**
	 * Le nom de l'objet merise.
	 */
	private String name;
	/**
	 * La description de l'objet merise.
	 */
	private String description;
	/**
	 * La liste des attributs.
	 */
	private List<Attribute> attributes;
	/**
	 * La liste des liens.
	 */
	private List<MeriseLink> meriseLinks;

	/**
	 * Le constructeur de la classe {@link MeriseObject}.
	 * 
	 * @param name
	 *            Le nom de l'objet merise
	 */
	public MeriseObject(String name) {
		this(name, "");
	}

	/**
	 * Le constructeur de la classe {@link MeriseObject}
	 * 
	 * @param name
	 *            Le nom de l'objet merise
	 * @param description
	 *            La description de l'objet merise
	 */
	public MeriseObject(String name, String description) {
		this.name = name;
		this.description = description;
		this.attributes = new ArrayList<Attribute>();
		this.meriseLinks = new ArrayList<MeriseLink>();
	}

	/**
	 * Ajoute un lien associé à l'objet.
	 * 
	 * @param meriseLink
	 *            Le lien à ajouter
	 */
	public void addLink(MeriseLink meriseLink) {
		meriseLinks.add(meriseLink);
	}

	/**
	 * Supprime un lien associé à l'objet.
	 * 
	 * @param meriseLink
	 *            Le lien à supprimer
	 */
	public void removeLink(MeriseLink meriseLink) {
		meriseLinks.remove(meriseLink);
	}

	/**
	 * Retourne un lien associé à l'objet.
	 * 
	 * @param index
	 *            L'index du lien
	 * @return le lien associé à l'objet
	 */
	public MeriseLink getLink(int index) {
		return meriseLinks.get(index);
	}

	/**
	 * Retourne la liste des liens.
	 * 
	 * @return la liste des liens
	 */
	public Iterable<MeriseLink> getLinks() {
		return new Iterable<MeriseLink>() {
			@Override
			public Iterator<MeriseLink> iterator() {
				return meriseLinks.iterator();
			}
		};
	}

	/**
	 * Retourne le nombre de liens.
	 * 
	 * @return le nombre de liens
	 */
	public int sizeLinks() {
		return meriseLinks.size();
	}

	/**
	 * Ajoute un attribut à l'objet.
	 * 
	 * @param attribute
	 *            un attribut
	 */
	public void addAttribute(Attribute attribute) {
		attributes.add(attribute);
	}

	/**
	 * Supprime un attribut à l'objet.
	 * 
	 * @param attribute
	 *            un attribut à l'objet
	 */
	public void removeAttribute(Attribute attribute) {
		attributes.remove(attribute);
	}

	/**
	 * Retourne un attribut de l'objet.
	 * 
	 * @param index
	 *            l'index de l'attribut
	 * @return un attribut de l'objet
	 */
	public Attribute getAttribute(int index) {
		return attributes.get(index);
	}

	/**
	 * Retourne la liste des attributs.
	 * 
	 * @return la liste des attributs
	 */
	public Iterable<Attribute> getAttributes() {
		return new Iterable<Attribute>() {
			@Override
			public Iterator<Attribute> iterator() {
				return attributes.iterator();
			}
		};
	}

	/**
	 * Supprime les attributs.
	 */
	public void clearAttributes() {
		attributes.clear();
	}

	/**
	 * Retourne le nombre d'attributs.
	 * 
	 * @return le nombre d'attributs
	 */
	public int sizeAttributes() {
		return attributes.size();
	}

	/**
	 * Retourne le nom de l'objet merise.
	 * 
	 * @return le nom de l'objet merise
	 */
	public String getName() {
		return name;
	}

	/**
	 * Modifie le nom de l'objet merise.
	 * 
	 * @param name
	 *            le nom de l'objet merise
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * Retourne la description de l'objet merise.
	 * 
	 * @return la description de l'objet merise
	 */
	public String getDescription() {
		return description;
	}

	/**
	 * Modifie la description de l'objet merise.
	 * 
	 * @param description
	 *            la description de l'objet merise
	 */
	public void setDescription(String description) {
		this.description = description;
	}
}
