/*
 * Copyright AnalyseSI, 2008
 * 
 * Loic DREUX
 * 
 * Ce logiciel est régi par la licence CeCILL soumise au droit français et
 * respectant les principes de diffusion des logiciels libres. Vous pouvez
 * utiliser, modifier et/ou redistribuer ce programme sous les conditions
 * de la licence CeCILL telle que diffusée par le CEA, le CNRS et l'INRIA 
 * sur le site "http://www.cecill.info".
 * 
 * En contrepartie de l'accessibilité au code source et des droits de copie,
 * de modification et de redistribution accordés par cette licence, il n'est
 * offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
 * seule une responsabilité restreinte pèse sur l'auteur du programme,  le
 * titulaire des droits patrimoniaux et les concédants successifs.
 * 
 * A cet égard  l'attention de l'utilisateur est attirée sur les risques
 * associés au chargement,  à l'utilisation,  à la modification et/ou au
 * développement et à la reproduction du logiciel par l'utilisateur étant 
 * donné sa spécificité de logiciel libre, qui peut le rendre complexe à 
 * manipuler et qui le réserve donc à des développeurs et des professionnels
 * avertis possédant  des  connaissances  informatiques approfondies.  Les
 * utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
 * logiciel à leurs besoins dans des conditions permettant d'assurer la
 * sécurité de leurs systèmes et ou de leurs données et, plus généralement, 
 * à l'utiliser et l'exploiter dans les mêmes conditions de sécurité. 
 * 
 * Le fait que vous puissiez accéder à cet en-tête signifie que vous avez 
 * pris connaissance de la licence CeCILL, et que vous en avez accepté les
 * termes.
 */
package org.eucalyptus.analysesi.plugin;

import java.io.File;
import java.io.FileFilter;
import java.net.MalformedURLException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eucalyptus.analysesi.db.DbManager;
import org.eucalyptus.egt.JarFileLoader;
import org.eucalyptus.egt.ManagerContainer;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Le gestionnaire de plugin.
 * 
 * @author Loic Dreux
 */
public class DefaultPluginManager implements PluginManager {
	// Le logger
	private static final Logger logger = LoggerFactory
			.getLogger(DefaultPluginManager.class);

	/**
	 * La liste des plugins installés
	 */
	private List<Plugin> installedPlugins;

	/**
	 * Le gestionnaire de base de données
	 */
	private DbManager dbManager;
	
	/**
	 * L'association Plugin -> ClassLoader.
	 */
	private Map<Plugin, ClassLoader> classLoaders;

	/**
	 * Le constructeur du {@link DefaultPluginManager}.
	 */
	public DefaultPluginManager(ManagerContainer managerContainer) {
		this.installedPlugins = new ArrayList<Plugin>();
		this.dbManager = (DbManager)managerContainer.getManager(DbManager.class);
		this.classLoaders = new HashMap<Plugin, ClassLoader>();
	}

	/**
	 * Retourne la liste des plugins installables.
	 * 
	 * @return la liste des plugins installables
	 */
	@Override
	public List<Plugin> getAvailablePlugins() {
		// TODO Ecrire listOfAvailablePlugins()
		return new ArrayList<Plugin>();
	}

	/* (non-Javadoc)
	 * @see org.eucalyptus.analysesi.plugin.PluginManager#getClassLoader(org.eucalyptus.analysesi.plugin.Plugin)
	 */
	@Override
	public ClassLoader getClassLoader(Plugin plugin) {
		return classLoaders.get(plugin);
	}

	/* (non-Javadoc)
	 * @see org.eucalyptus.analysesi.plugin.PluginManager#getClassLoaders()
	 */
	@Override
	public Collection<ClassLoader> getClassLoaders() {
		return classLoaders.values();
	}

	/**
	 * Retourne la liste des plugins installés
	 * 
	 * @return la liste des plugins intallésilefilter
	 */
	@Override
	public List<Plugin> getInstalledPlugins() {
		return this.installedPlugins;
	}

	/**
	 * Charge les plugins.
	 * 
	 * @param folders
	 *            Les répertoires contenant les plugins
	 */
	@Override
	public void loadPlugins(List<File> folders) {
		installedPlugins = new ArrayList<Plugin>();

		logger.debug("Chargement des plugins");

		// Création de la fabrique de plugin
		PluginFactory factory = new PluginFactory(dbManager, folders);

		// On parcours les répertoire contenant les plugins
		for (File folder : folders) {
			logger
					.debug("== Parcours des fichiers présent dans le répertoire : "
							+ folder.getAbsolutePath());

			// On parcours les descripteurs
			for (File descriptor : folder.listFiles(new XMLFileFilter())) {
				logger.debug("==== Chargement du plugin : "
						+ descriptor.getName());

				try {
					// Création du plugin
					Plugin plugin = factory.buildPlugin(descriptor);
					
					// Création du ClassLoader
					JarFileLoader classLoader = new JarFileLoader(plugin.getJarfile());
					classLoaders.put(plugin, classLoader);
					
					// Chargement et ajout du plugin
					plugin.load();
					installedPlugins.add(plugin);
					logger.info("Plugin Loaded : " + plugin.getName());
				} catch (PluginException e) {
					logger.error(e.getLocalizedMessage(), e);
				} catch (MalformedURLException e) {
					logger.error(e.getLocalizedMessage(), e);
				}
			}
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eucalyptus.analysesi.plugin.PluginManager#removePlugin(org.eucalyptus
	 * .analysesi.plugin.Plugin)
	 */
	@Override
	public void removePlugin(Plugin plugin) {
		installedPlugins.remove(plugin);
	}

	/**
	 * Ce filtre n'accepte que les fichiers XML.
	 * 
	 * @author Loic Dreux
	 */
	private class XMLFileFilter implements FileFilter {

		/*
		 * (non-Javadoc)
		 * 
		 * @see java.io.FileFilter#accept(java.io.File)
		 */
		@Override
		public boolean accept(File pathname) {
			return pathname.getAbsolutePath().endsWith(".xml");
		}

	}
}
