/*
 * Copyright AnalyseSI, 2008
 * 
 * Loic DREUX
 * 
 * Ce logiciel est régi par la licence CeCILL soumise au droit français et
 * respectant les principes de diffusion des logiciels libres. Vous pouvez
 * utiliser, modifier et/ou redistribuer ce programme sous les conditions
 * de la licence CeCILL telle que diffusée par le CEA, le CNRS et l'INRIA 
 * sur le site "http://www.cecill.info".
 * 
 * En contrepartie de l'accessibilité au code source et des droits de copie,
 * de modification et de redistribution accordés par cette licence, il n'est
 * offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
 * seule une responsabilité restreinte pèse sur l'auteur du programme,  le
 * titulaire des droits patrimoniaux et les concédants successifs.
 * 
 * A cet égard  l'attention de l'utilisateur est attirée sur les risques
 * associés au chargement,  à l'utilisation,  à la modification et/ou au
 * développement et à la reproduction du logiciel par l'utilisateur étant 
 * donné sa spécificité de logiciel libre, qui peut le rendre complexe à 
 * manipuler et qui le réserve donc à des développeurs et des professionnels
 * avertis possédant  des  connaissances  informatiques approfondies.  Les
 * utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
 * logiciel à leurs besoins dans des conditions permettant d'assurer la
 * sécurité de leurs systèmes et ou de leurs données et, plus généralement, 
 * à l'utiliser et l'exploiter dans les mêmes conditions de sécurité. 
 * 
 * Le fait que vous puissiez accéder à cet en-tête signifie que vous avez 
 * pris connaissance de la licence CeCILL, et que vous en avez accepté les
 * termes.
 */
package org.eucalyptus.analysesi.plugin;

import java.io.File;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Un plugin pour AnalyseSI.
 * 
 * Un plugin est spécifié par un descripteur au format XML.
 * 
 * Voici un exemple <code>
 * 	<plugin>
 * 		<name>nom</name>
 * 		<version>1.0</version>
 * 		<type>DATABASE_DRIVER</type>
 * 		<jar>mon_fichier.jar</jar>
 * 		<properties>
 * 			<property key="mykey" value="myvalue" />
 * 		</properties>
 * 	</plugin>
 * </code>
 * 
 * @author Loic Dreux
 */
public abstract class Plugin {
	// Le logger
	@SuppressWarnings("unused")//$NON-NLS-1$
	private static final Logger logger = LoggerFactory.getLogger(Plugin.class);

	/**
	 * Le nom du plugin
	 */
	private String name;

	/**
	 * Le type du plugin.
	 */
	private PluginType type;

	/**
	 * La version du plugin.
	 */
	private String version;

	/**
	 * Le nom du fichier jar.
	 */
	private File jarfile;

	/**
	 * La description du plugin
	 */
	private String description;

	/**
	 * Le constructeur d'un {@link Plugin}.
	 * 
	 * @param name
	 *            Le nom du plugin
	 * @param description
	 * 	          La description du plugin
	 * @param version
	 *            La version du plugin
	 * @param type
	 *            Le type de plugin
	 * @param jarfile
	 *            Le fichier jar associé
	 */
	protected Plugin(String name, String description, String version, PluginType type, File jarfile) {
		this.name = name;
		this.description = description;
		this.version = version;
		this.jarfile = jarfile;
		this.type = type;
	}

	/**
	 * Retourne le nom du fichier jar.
	 * 
	 * @return the jarfile
	 */
	public File getJarfile() {
		return jarfile;
	}

	/**
	 * Retourne le nom du plugin.
	 * 
	 * @return le nom du plugin
	 */
	public String getName() {
		return name;
	}

	/**
	 * Retourne le type du plugin.
	 * 
	 * @return le type du plugin
	 */
	public PluginType getType() {
		return type;
	}

	/**
	 * Retourne le numéro de version.
	 * 
	 * @return the version
	 */
	public String getVersion() {
		return version;
	}

	/**
	 * Charge le plugin.
	 * 
	 * @throws PluginException
	 */
	public abstract void load() throws PluginException;

	/**
	 * Retourne la description.
	 * 
	 * @return the description
	 */
	public String getDescription() {
		return description;
	}
}
