/*
 * Copyright AnalyseSI, 2008
 * 
 * Loic DREUX
 * 
 * Ce logiciel est régi par la licence CeCILL soumise au droit français et
 * respectant les principes de diffusion des logiciels libres. Vous pouvez
 * utiliser, modifier et/ou redistribuer ce programme sous les conditions
 * de la licence CeCILL telle que diffusée par le CEA, le CNRS et l'INRIA 
 * sur le site "http://www.cecill.info".
 * 
 * En contrepartie de l'accessibilité au code source et des droits de copie,
 * de modification et de redistribution accordés par cette licence, il n'est
 * offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
 * seule une responsabilité restreinte pèse sur l'auteur du programme,  le
 * titulaire des droits patrimoniaux et les concédants successifs.
 * 
 * A cet égard  l'attention de l'utilisateur est attirée sur les risques
 * associés au chargement,  à l'utilisation,  à la modification et/ou au
 * développement et à la reproduction du logiciel par l'utilisateur étant 
 * donné sa spécificité de logiciel libre, qui peut le rendre complexe à 
 * manipuler et qui le réserve donc à des développeurs et des professionnels
 * avertis possédant  des  connaissances  informatiques approfondies.  Les
 * utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
 * logiciel à leurs besoins dans des conditions permettant d'assurer la
 * sécurité de leurs systèmes et ou de leurs données et, plus généralement, 
 * à l'utiliser et l'exploiter dans les mêmes conditions de sécurité. 
 * 
 * Le fait que vous puissiez accéder à cet en-tête signifie que vous avez 
 * pris connaissance de la licence CeCILL, et que vous en avez accepté les
 * termes.
 */
package org.eucalyptus.analysesi.plugin;

import java.io.File;
import java.io.IOException;
import java.util.List;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.eucalyptus.analysesi.db.DbManager;
import org.eucalyptus.analysesi.gui.Messages;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.xml.sax.SAXException;

/**
 * La fabrique de plugin.
 * 
 * @author Loic Dreux
 */
public class PluginFactory {
	// Le logger
	@SuppressWarnings("unused")//$NON-NLS-1$
	private static final Logger logger = LoggerFactory
			.getLogger(PluginFactory.class);

	/**
	 * Le gestionnaire de base de données
	 */
	private DbManager dbManager;

	/**
	 * La liste des répertoires contenant les plugins
	 */
	private List<File> pluginFolders;

	/**
	 * Constructeur de {@link PluginFactory}.
	 * 
	 * @param dbManager
	 */
	public PluginFactory(DbManager dbManager, List<File> pluginFolders) {
		this.dbManager = dbManager;
		this.pluginFolders = pluginFolders;
	}

	/**
	 * Fabrique un plugin à partir d'un fichier
	 * 
	 * @throws PluginException
	 *             En cas d'erreur
	 * @throws NullPointerException
	 *             si file est nul
	 * @param file
	 *            Le fichier "descriptor"
	 * @return Le nouveau plugin
	 */
	public Plugin buildPlugin(File file) throws PluginException {
		if (file == null)
			throw new NullPointerException("File cannot be null"); //$NON-NLS-1$

		try {
			// Déclaration du plugin et de son fichier jar
			Plugin plugin = null;
			File jarfile = null;

			// Création des instance
			SAXParserFactory factory = SAXParserFactory.newInstance();
			SAXParser parser = factory.newSAXParser();
			DescriptorHandler handler = new DescriptorHandler();

			// On parse
			parser.parse(file, handler);
			
			// On vérifie les données du handler
			if (handler.getName() == null) {
				throw new PluginException(Messages.getString("PluginFactory.nameError")); //$NON-NLS-1$
			} else if (handler.getDescription() == null) {
				throw new PluginException(Messages.getString("PluginFactory.descriptionError"));  //$NON-NLS-1$
			} else if (handler.getVersion() == null) {
				throw new PluginException(Messages.getString("PluginFactory.versionError"));  //$NON-NLS-1$
			} else if (handler.getType() == null) {
				throw new PluginException(Messages.getString("PluginFactory.typeError"));  //$NON-NLS-1$
			} else if (handler.getJarfile() == null) {
				throw new PluginException(Messages.getString("PluginFactory.jarfileError"));  //$NON-NLS-1$
			}

			// On recherche le fichier
			for (File folder : this.pluginFolders) {
				File testJarfile = new File(folder.getAbsolutePath()
						+ File.separator + handler.getJarfile());
				if (testJarfile.exists()) {
					jarfile = testJarfile;
				}
			}
			
			// On vérifie que le fichier jar a été trouvé
			if (jarfile == null) {
				throw new PluginException(Messages.getString("PluginFactory.jarfileNotFoundError")); //$NON-NLS-1$
			}
			
			// Par défaut la requête sql si elle n'est pas défini est SELECT 1
			String sqlTestQuery = handler.getProperty("sqlTestQuery");
			if (sqlTestQuery == null)
				sqlTestQuery = "SELECT 1";

			// Construction du plugin en fonction du type
			if (handler.getType().equals(PluginType.DATABASE_DRIVER)) {
				plugin = new DatabaseDriverPlugin(handler.getName(), handler
						.getDescription(), handler.getVersion(), jarfile,
						handler.getProperty("driverClass"), handler //$NON-NLS-1$
								.getProperty("urlStart"), sqlTestQuery, dbManager); //$NON-NLS-1$
			} else {
				throw new PluginException(Messages.getString("PluginFactory.typeUnknownError")); //$NON-NLS-1$
			}

			return plugin;
		} catch (PluginException e) {
			throw e;
		} catch (SAXException e) {
			throw new PluginException(Messages.getString("PluginFactory.saxexceptionError"), e); //$NON-NLS-1$
		} catch (IOException e) {
			throw new PluginException(Messages.getString("PluginFactory.ioexceptionError"), e); //$NON-NLS-1$
		} catch (ParserConfigurationException e) {
			throw new PluginException(Messages.getString("PluginFactory.parserconfigurationexceptionError"), e); //$NON-NLS-1$
		}
	}
}
