/*
 * Copyright AnalyseSI, 2008
 * 
 * Loic DREUX
 * 
 * Ce logiciel est régi par la licence CeCILL soumise au droit français et
 * respectant les principes de diffusion des logiciels libres. Vous pouvez
 * utiliser, modifier et/ou redistribuer ce programme sous les conditions
 * de la licence CeCILL telle que diffusée par le CEA, le CNRS et l'INRIA 
 * sur le site "http://www.cecill.info".
 * 
 * En contrepartie de l'accessibilité au code source et des droits de copie,
 * de modification et de redistribution accordés par cette licence, il n'est
 * offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
 * seule une responsabilité restreinte pèse sur l'auteur du programme,  le
 * titulaire des droits patrimoniaux et les concédants successifs.
 * 
 * A cet égard  l'attention de l'utilisateur est attirée sur les risques
 * associés au chargement,  à l'utilisation,  à la modification et/ou au
 * développement et à la reproduction du logiciel par l'utilisateur étant 
 * donné sa spécificité de logiciel libre, qui peut le rendre complexe à 
 * manipuler et qui le réserve donc à des développeurs et des professionnels
 * avertis possédant  des  connaissances  informatiques approfondies.  Les
 * utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
 * logiciel à leurs besoins dans des conditions permettant d'assurer la
 * sécurité de leurs systèmes et ou de leurs données et, plus généralement, 
 * à l'utiliser et l'exploiter dans les mêmes conditions de sécurité. 
 * 
 * Le fait que vous puissiez accéder à cet en-tête signifie que vous avez 
 * pris connaissance de la licence CeCILL, et que vous en avez accepté les
 * termes.
 */
package org.eucalyptus.analysesi.save;

import java.util.ArrayList;
import java.util.List;

/**
 * Le gestionnaire de sauvegarde d'AnalyseSI par défaut.
 * 
 * @author Loic Dreux
 */
public class DefaultSaveManager implements SaveManager {
	/**
	 * Le fichier sauvegarder.
	 */
	private AnalyseSIFile currentSaveFile;

	/**
	 * Indique si le fichier courant a subi des modifications après sa dernière
	 * sauvegarde.
	 */
	private boolean isModified;
	
	/**
	 * La liste des formats supportés
	 */
	private List<Format> supportedFormats;

	/**
	 * Le constructeur {@link DefaultSaveManager}.
	 */
	public DefaultSaveManager() {
		this.currentSaveFile = null;
		this.isModified = false;
		this.supportedFormats = new ArrayList<Format>();
		
		// Ajout des formats supportés
		supportedFormats.add(new HibiscusFormat());
		supportedFormats.add(new AnalyseSI063Format());
		// TODO Ajouter les formats plugins
	}

	/**
	 * Indique que le fichier courant à subi des modification.
	 */
	@Override
	public void fileModified() {
		this.isModified = true;
	}

	/**
	 * Retourne le fichier de sauvegarde ou null si aucun fichier
	 * n'a été modifié.
	 * 
	 * @return the currentSaveFile
	 */
	@Override
	public AnalyseSIFile getCurrentAnalyseSIFile() {
		return currentSaveFile;
	}

	/**
	 * Indique si le fichier courant a subi des modifications après sa dernière
	 * sauvegarde.
	 * 
	 * @return true si le fichier à été modifié, false sinon
	 */
	@Override
	public boolean isModified() {
		return isModified;
	}
	
	/**
	 * Sauvegarde le model dans un fichier AnalyseSI.
	 * 
	 * @param analyseSIModel
	 *            Le model
	 * @param analyseSIFile
	 *            Le fichier AnalyseSI
	 */
	@Override
	public void saveModel(AnalyseSIModel analyseSIModel, AnalyseSIFile analyseSIFile) throws SaveModelException, SaveModelNotSupportedException {
		// Récupération du format
		Format format = analyseSIFile.getFormat();
		
		// Sauvegarde du modèle
		format.saveModel(analyseSIModel, analyseSIFile.getFile());
	}

	/**
	 * Charge le model depuis un fichier AnalyseSI.
	 * 
	 * @param analyseSIFile
	 *            Le fichier AnalyseSI
	 * @return le model
	 */
	@Override
	public AnalyseSIModel loadModel(AnalyseSIFile analyseSIFile) throws LoadModelException, LoadModelNotSupportedException {
		// Récupération du format
		Format format = analyseSIFile.getFormat();
		
		// Chargement du modèle
		return format.loadModel(analyseSIFile.getFile());
	}

	/**
	 * Retourne la liste des formats supportés.
	 * 
	 * @return la liste des formats supportés
	 */
	@Override
	public List<Format> getSupportedFormats() {
		return supportedFormats;
	}

	/**
	 * Retourne le format par rapport à son nom.
	 * 
	 * @param name
	 *            Le nom du format
	 * @return le format qui possède le nom donné en paramètre
	 * @return null si aucun format ne correspond au nom
	 */
	@Override
	public Format getFormatByName(String name) {
		for (Format format: getSupportedFormats()) {
			if (format.getName().equals(name))
				return format;
		}
		return null;
	}

	/**
	 * Retourne le format par rapport à son extension.
	 * 
	 * @param extension
	 *            L'extension du format
	 * @return le format qui possède l'extension donnée en paramètre
	 * @return null si aucun format ne correspond à l'extension
	 */
	@Override
	public Format getFormatByExtension(String extension) {
		for (Format format: getSupportedFormats()) {
			if (format.getFileExtension().equals(extension))
				return format;
		}
		return null;
	}
}
