/*
 * Copyright AnalyseSI, 2008
 * 
 * Loic DREUX
 * 
 * Ce logiciel est régi par la licence CeCILL soumise au droit français et
 * respectant les principes de diffusion des logiciels libres. Vous pouvez
 * utiliser, modifier et/ou redistribuer ce programme sous les conditions
 * de la licence CeCILL telle que diffusée par le CEA, le CNRS et l'INRIA 
 * sur le site "http://www.cecill.info".
 * 
 * En contrepartie de l'accessibilité au code source et des droits de copie,
 * de modification et de redistribution accordés par cette licence, il n'est
 * offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
 * seule une responsabilité restreinte pèse sur l'auteur du programme,  le
 * titulaire des droits patrimoniaux et les concédants successifs.
 * 
 * A cet égard  l'attention de l'utilisateur est attirée sur les risques
 * associés au chargement,  à l'utilisation,  à la modification et/ou au
 * développement et à la reproduction du logiciel par l'utilisateur étant 
 * donné sa spécificité de logiciel libre, qui peut le rendre complexe à 
 * manipuler et qui le réserve donc à des développeurs et des professionnels
 * avertis possédant  des  connaissances  informatiques approfondies.  Les
 * utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
 * logiciel à leurs besoins dans des conditions permettant d'assurer la
 * sécurité de leurs systèmes et ou de leurs données et, plus généralement, 
 * à l'utiliser et l'exploiter dans les mêmes conditions de sécurité. 
 * 
 * Le fait que vous puissiez accéder à cet en-tête signifie que vous avez 
 * pris connaissance de la licence CeCILL, et que vous en avez accepté les
 * termes.
 */
package org.eucalyptus.analysesi.save;

import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

import org.eucalyptus.analysesi.model.MeriseModel;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import com.thoughtworks.xstream.XStream;

/**
 * 
 * @test EucalyptusFormat.saveMeriseModel() : Test avec un modèle dont le
 *       dictionnaire contient 5 attributs
 * @test EucalyptusFormat.saveMeriseModel() : Test avec un modèle et un
 *       dictionnaire vide
 * @test EucalyptusFormat.saveMeriseModel() : Test avec un modèle dont le
 *       dictionnaire contient 5 attributs
 * @test EucalyptusFormat.loadMeriseModel() : Test avec un modèle null
 *       (NullPointerException).
 * @test EucalyptusFormat.loadMeriseModel() : Test avec un modèle et un
 *       dictionnaire vide.
 * @test EucalyptusFormat.loadMeriseModel() : Test avec un modèle dont le
 *       dictionnaire contient 5 attributs.
 * @test EucalyptusFormat.loadMeriseModel() : Test avec un modèle dont le type
 *       de base de données n'est pas pris en compte.
 * @author Loic Dreux
 */
public class EucalyptusFormatTest {

	/**
	 * Le gestionnaire de sauvegarde
	 */
	private SaveManager saveManager;

	/**
	 * Le fichier de destination
	 */
	private File saveFile;

	/**
	 * Le fichier analyseSI
	 */
	private AnalyseSIFile analyseSIFile;

	/**
	 * Le modèle merise
	 */
	private MeriseModel meriseModel;

	/*
	 * (non-Javadoc)
	 * 
	 * @see junit.framework.TestCase#setUp()
	 */
	@Before
	public void setUp() throws IOException {
		saveManager = new DefaultSaveManager();
		saveFile = File.createTempFile("tmpfile", ".tmp");
		analyseSIFile = new AnalyseSIFile(saveFile, new HibiscusFormat());
		meriseModel = new MeriseModel();

		saveFile.delete();
		assertTrue(!saveFile.exists());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see junit.framework.TestCase#tearDown()
	 */
	@After
	public void tearDown() {
		saveFile.delete();
		assertTrue(!saveFile.exists());
	}

	/*
	 * ============================================= SaveMeriseModel
	 */

	/**
	 * Test avec un modèle null (NullPointerException)
	 * 
	 * @throws SaveModelNotSupportedException
	 * @throws SaveModelException
	 */
	@Test(expected = NullPointerException.class)
	public void testSaveMeriseModelNullPointerException() throws Exception {
		saveManager.saveModel(new AnalyseSIModel() {
			@Override
			public MeriseModel getMeriseModel() {
				return null;
			}
		}, analyseSIFile);
	}

	/**
	 * Test avec un modèle et un dictionnaire vide
	 */
	@Test
	public void testSaveMeriseModelEmptyDictionary() throws Exception {		
		saveManager.saveModel(new AnalyseSIModel() {
			@Override
			public MeriseModel getMeriseModel() {
				return meriseModel;
			}
		}, analyseSIFile);
		
		// On vérifie que le fichier existe bien
		assertTrue(analyseSIFile.getFile().exists());
		
		// On vérifie le contenue du fichier Zip
		ZipFile zipFile = new ZipFile(analyseSIFile.getFile());
		ZipEntry zipEntry = zipFile.getEntry("meriseModel.xml");
		InputStream inputStream = zipFile.getInputStream(zipEntry);
		
		// On charge le modèle
		XStream xstream = new XStream();
		MeriseModel meriseModel = (MeriseModel)xstream.fromXML(inputStream);
		
		// On vérifie l'objet modèle
		assertNotNull(meriseModel);
		assertNotNull(meriseModel.getDictionary());
		
		System.out.print(analyseSIFile.getFile().toString());
	}

	/**
	 * Test avec un modèle dont le dictionnaire contient 5 attributs
	 */
	@Test
	public void testSaveMeriseModel5AttributesInDictionary() {
		//assertTrue(false);
	}

	/*
	 * ============================================= loadMeriseModel
	 */

	/**
	 * Test avec un modèle null (NullPointerException).
	 */
	@Test
	public void testLoadMeriseModelNullPointerException() {
		//assertTrue(false);
	}

	/**
	 * Test avec un modèle et un dictionnaire vide.
	 */
	@Test
	public void testLoadMeriseModelEmptyDictionary() {
		//assertTrue(false);
	}

	/**
	 * Test avec un modèle dont le dictionnaire contient 5 attributs.
	 */
	@Test
	public void testLoadMeriseModel5AttributesInDictionary() {
		//assertTrue(false);
	}

	/**
	 * Test avec un modèle dont le type de base de données n'est pas pris en
	 * compte.
	 */
	@Test
	public void testLoadMeriseModelDatabaseClassNotFound() {
		//assertTrue(false);
	}
}